 //*******************************************************************************
 //*                                                                             *
 //*       FABLIST.BAS - inspired by RWR//s LZLIST                                *
 //*                                                                             *
 //*       prints a text file on a laserjet, creates a //greebar// effect by       *
 //*       printing shaded bars on page. prints direct through BIOS              *
 //*                                                                             *
 //*       prints a header including filename, date and page number              *
 //*       and adds line numbers                                                 *
 //*                                                                             *
 //*       reads filename from command line or requests it                       *
 //*                                                                             *
 //*       accepts parameters to adjust defaults                                 *
 //*                                                                             *
 //*       DEFAULTS:               PARAMETER:              RESULT:               *
 //*       portrait                \NOPORT                 prints landscape      *
 //*       compressed              \NOCOMPR                normal courier        *
 //*       IBM symbol set          \NOIBM                  HP symbol set         *
 //*       8 lines per inch        \6LPI                   6 lines per inch      *
 //*       use LPT1:               \LPT2                   use LPT2:             *
 //*       bars two lines thick    \LINES                  bars one line thick   *
 //*       or three lines thick                                                  *
 //*       at 6lpi                                                               *
 //*                                                                             *
 //*       parameters can be used in any combination or order                    *
 //*                                                                             *
 //*       12/25/90        must be compiled with PDQ and my ADVBAS-based         *
 //*                       subroutines                                           *
 //*                                                                             *
 //*                                                                             *
 //*       Copyright 1990, Richard G. Minutillo                                                       *
 //*                                                                             *
 //*******************************************************************************
 //
 DEFINT A-Z
 OPTION BASE 1                                                   //for PDQ
 //$INCLUDE: //PDQDECL.BAS//                                        //for PDQ
 //$INCLUDE: //MYDECL.BAS//                                         //for me
 //
 DIM ch AS STRING * 1                                            //allow single character read from file
 //
 DIM es AS STRING * 1                                            //will be defined as CHR$(27)
 DIM cr AS STRING * 1                                            //will be defined as CHR$(13)
 DIM lf AS STRING * 1                                            //will be defined as CHR$(10)
 DIM ff AS STRING * 1                                            //will be defined as CHR$(12)
 //
 es = CHR$(27)
 cr = CHR$(13)
 lf = CHR$(10)
 ff = CHR$(12)
 //
 //       these are default values
 //
 startwid = 8                                                    //allows for line numbers and indents
 compressed = -1                                                 //logical flag for typeface
 portrait = -1                                                   //logical flag for orientation
 lpi = 8                                                         //lines per inch
 sline = 2                                                       //number of shaded lines
 lpt = 1                                                         //for lpt1:
 //
 PRINT 'FABLIST from the Fine Arts BluesBand & Poetry Press'     //say hello
 //
 //       read command$ for filename and any altered defaults
 //
 CALL SetDelimitChar(32)                                         //now use a blank space as a delimiter
 Work$ = COMMAND$                                                //get the command line arguments
 DO
         Temp$ = UCASE$(PDQParse$(Work$))                        //get the next argument
         IF LEN(Temp$) = 0 THEN EXIT DO                          //if there is one
         SELECT CASE Temp$                                       //then look for legal switches
                 CASE '\NOIBM'
                         noibm = -1
                 CASE '\NOPORT'
                         portrait = 0
                 CASE '\NOCOMPR'
                         compressed = 0
                 CASE '\6LPI'
                         lpi = 6
 \PROG\PDQ\FABLIST.BAS                       Friday, June 6th at 6:26 pm                                                Page 2

                 CASE '\LPT2'
                         lpt = 2
                 CASE '\LINES'
                         sline = 1
                 CASE ELSE                                       //or else assume that
                         filename$ = Temp$                       //it//s the filename
         END SELECT
 LOOP
 //
 IF compressed AND portrait THEN                                 //this is a brute force
         wid = 133                                               //method of setting
         lengthinhalfinches = 20                                 //other print parameters
         widthinhalfinches = 16                                  //using half inches to
 ELSEIF compressed AND NOT portrait THEN                         //allow legal results
         wid = 176                                               //since PDQ doesn//t allow
         lengthinhalfinches = 15                                 //floating point and so
         widthinhalfinches = 21                                  //half-inches are verboten
 ELSEIF NOT compressed AND portrait THEN                         //and adjustimg a few max
         wid = 80                                                //parameters to compensate
         lengthinhalfinches = 20                                 //for the integer divisions
         widthinhalfinches = 16                                  //especially in the
 ELSEIF NOT compressed AND NOT portrait THEN                     //landscape width, which is
         wid = 105                                               //one less than the laser
         lengthinhalfinches = 15                                 //will print but is the same
         widthinhalfinches = 21                                  //as the shading width
 END IF
 //
 IF lpi = 6 AND sline <> 1 THEN sline = 3                        //shade 3 lines at a time at 6lpi
 //
 linedots = 720 \ lpi                                            //number of decipoints/per/line
 leng = (lengthinhalfinches * lpi) \ 2                           //number of lines per page
 wdots = (300 * widthinhalfinches) \ 2                           //shade width in dots (@300 dpi)
 //
 //       check filename from command$
 //
 IF LEN(filename$) = 0 THEN                                      //if no filename
         DO                                                              //if no filename then get one
                 PRINT 'Enter Filename: ';                               //give us a prompt
                 CALL PDQInput(filename$)                                //get one
         LOOP WHILE LEN(filename$) = 0                                   //get something, anyway
 END IF
 //
 filename$ = filename$ + CHR$(0)                                 //ascii-z for checking
 CALL FINDFIRSTF(filename$, 0, ercd)                             //check for its existence
 filename$ = RTRIM$(filename$)                                   //take off ascii-z
 IF ercd THEN                                                    //if it doesn//t exist then
         PRINT 'File Not Found: ' + filename$                    //print error
         STOP                                                    //then go away
 ELSE
         PRINT 'Printing File: ' + filename$                     //otherwise signal start of print
 END IF
 //
 Work$ = es + 'E': GOSUB Pit                                     //start with reset
 //
 IF NOT portrait THEN
         Work$ = es + '&l1O': GOSUB Pit                          //set landscape if required
 END IF
 //
 Work$ = es + '&l' + STR$(lpi) + 'D': GOSUB Pit                  //set 8 lpi
 IF NOT noibm THEN Work$ = es + '(10U': GOSUB Pit                //set IBM symbol set
 //
 IF compressed THEN
         Work$ = es + '(s0p16.66H': GOSUB Pit                    //set compressed if required
 END IF
 //
 header$ = STRING$(wid, 32)                                      //start build of  header line
 MID$(header$, 1) = '  File: ' + filename$                       //insert file name
 CALL GOODDATE(wd$, mo$, dt$)                                    //format date
 stamp$ = wd$ + ', ' + mo$ + ' ' + dt$ + ' at ' + GOODTIME$      //format date & time stamp
 MID$(header$, (wid \ 2) - (LEN(stamp$) \ 2)) = stamp$           //insert date and time stamp in middle
 //
 GOSUB ShadeIt                                                   //create shading macro and invoke as overlay
 GOSUB NewPage: GOSUB NewLnum:                                   //increment page & line numbers to '1'
 //
 OPEN filename$ FOR BINARY AS 1                                  //open for binary to allow bit-wise reading
 //
 Work$ = header$ + cr + lf: GOSUB Pit                            //print header line
 Work$ = cr + lf: GOSUB Pit                                      //skip a line
 \PROG\PDQ\FABLIST.BAS                       Friday, June 6th at 6:26 pm                                                Page 3

 Work$ = lnum$: GOSUB Pit                                        //print first line number
 wd = startwid: ln = 3: newln = 0                                //reset page & width counters and new line cue
 //
 //       main loop reads input file one character at a time
 //
 DO WHILE NOT EOF(1)
         GET 1, , ch
         IF newln = 1 THEN                                       //if line counter has been tripped
                 IF ln > leng THEN                               //see if we need to start new physical page
                         GOSUB NewPage                           //advance the page number
                         Work$ = header$ + cr + lf: GOSUB Pit    //print header on the new page
                         Work$ = cr + lf: GOSUB Pit              //skip a line
                         ln = 3                                  //and reset the lines-on-page counter
                 END IF
                 Work$ = lnum$: GOSUB Pit:                       //print next line number
                 wd = startwid: newln = 0                        //reset width counter and new line cue
         END IF
 //
         IF ch = cr THEN                                         //if byte is carriage return we assume end of line
                 GET #1, , ch                                    //so we get the matching line feed
                 ln = ln + 1: newln = 1                          //advance page counter and turnon newline cue
                 GOSUB NewLnum                                   //and advance the line number
                 Work$ = cr + lf: GOSUB Pit                      //and print the cr and lf
 //
         ELSEIF ch = ff THEN                                     //if the byte is a literal form feed
                 ln = 1: newln = 1                               //reset the page line counter and turn on the newline cue
                 GOSUB NewLnum                                   //advance the line number counter
                 Work$ = ff: GOSUB Pit                           //and print literal form feed
         ELSE
                 Work$ = ch: GOSUB Pit                           //in all other cases we print the character
                 wd = wd + 1                                     //advance the line width counter
                 IF wd = wid THEN                                //if we//re at the end of the line
                         Work$ = cr + lf: GOSUB Pit              //and print the cr and lf
                         ln = ln + 1                             //advance the lines-on-page counter
                         IF ln > leng THEN                       //and check to see if we//re at the end of the page
                                 GOSUB NewPage                   //and advance the page number
                                 Work$ = header$ + cr + lf
                                 GOSUB Pit                       //print header on the new page
                                 Work$ = cr + lf: GOSUB Pit      //skip a line
                                 ln = 3                          //and reset lines-on-page counter
                                 Work$ = SPACE$(startwid + 4)    //blanks where the line number would be
                                 GOSUB Pit: wd = startwid + 5    //print the blanks and indent the width counter
                         END IF
                         //                                       //otherwise we just set the
                         Work$ = SPACE$(startwid + 4)            //blanks where the line number would be
                         GOSUB Pit: wd = startwid + 5            //print the blanks and indent the width counter
                 END IF
         END IF
 LOOP                                                            //end-of-loop = end-of-file
 //
 Work$ = CHR$(12): GOSUB Pit                                     //eject the last page
 Work$ = es + 'E': GOSUB Pit                                     //reset the printer again
 END                                                             //end of program
 //
 //               SUBROUTINES
 //               -----------
 //
 //       Pit: prints Work$ one character at a time through the BIOS
 //
 Pit:
 FOR X = 1 TO LEN(Work$)
         Dummy = BIOSPrint(ASC(MID$(Work$, X, 1)), lpt)
         IF Dummy THEN
                 //PRINT 'Printer Error! Check it and Try Again!'
                 //STOP
         END IF
 NEXT X
 RETURN
 //
 //       NewLnum: advances the line number and formats it for printing
 //
 NewLnum:
 lnum = lnum + 1
 lnum$ = STR$(lnum)
 lnum$ = STRING$(6 - LEN(lnum$), CHR$(32)) + lnum$ + ': '
 RETURN
 //
 //       NewPage: advances the page number, formats it &
 \PROG\PDQ\FABLIST.BAS                       Friday, June 6th at 6:26 pm                                                Page 4

 //       inserts in the header string at the right margin
 //
 NewPage:
 pnum = pnum + 1
 pnum$ = STR$(pnum)
 pnum$ = 'Page ' + pnum$
 MID$(header$, wid - LEN(pnum$) + ABS(portrait)) = pnum$         //an adjustment for neatness
 RETURN
 //
 //       ShadeIt: defines and invokes line shading macro
 //
 ShadeIt:
 Work$ = es + '&f1Y': GOSUB Pit                                          //specify macro ID
 Work$ = es + '&f0X': GOSUB Pit                                          //start macro def
 Work$ = es + '&f0S': GOSUB Pit                                          //store cursor position
 stepval = linedots * sline * 2
 FOR vert = linedots * sline TO (linedots * leng - 1) STEP stepval       //define bar start & stop
         Work$ = es + '&a' + STR$(vert) + 'V':   GOSUB Pit               //position cursor
         Work$ = es + '*c' + STR$(wdots) + 'A': GOSUB Pit                //set shade width
         Work$ = es + '*c' + STR$(linedots * sline) + 'V': GOSUB Pit     //set shade height
         Work$ = es + '*c10G': GOSUB Pit                                 //set shade degree
         Work$ = es + '*c2P': GOSUB Pit                                  //print shade
 NEXT
 Work$ = es + '&f1S': GOSUB Pit                                  //restore cursor position
 Work$ = es + '&f1X': GOSUB Pit                                  //end macro definition
 //
 Work$ = es + '&f4X': GOSUB Pit                                  //enable macro as overlay
 RETURN


