//******************************************************************************
//*                                                                            *
//*       FABPRINT.DPR - a new Windows(TM) version of FABList                  *
//*                                                                            *
//*       totally free software (and worth every penny) from                   *
//*       the homestyle kitchens of the Fine-Arts Bluesband.                   *
//*       Copyright 2003 by R.G. Minutillo.                                    *
//*                                                                            *
//*       Depends on the WINSPOOL direct printing, bypassing the               *
//*       Windows(TM) drivers, so be warned!                                   *
//*                                                                            *
//*       see the PrintHelp procedure for sytnax and options                   *
//*                                                                            *
//*       Summer, 2003                                                         *
//*                                                                            *
//******************************************************************************
//*       original comments:                                                   *
//******************************************************************************
//*                                                                            *
//*       FABLIST.BAS - inspired by RWR's LZLIST                               *
//*                                                                            *
//*       prints a text file on a laserjet, creates a 'greenbar' effect by     *
//*       printing shaded bars on page. prints direct through BIOS             *
//*                                                                            *
//*       prints a header including filename, date and page number             *
//*       and adds line numbers                                                *
//*                                                                            *
//*       reads filename from command line or requests it                      *
//*                                                                            *
//*       accepts parameters to adjust defaults                                *
//*                                                                            *
//*       DEFAULTS:               PARAMETER:              RESULT:              *
//*       portrait                \NOPORT                 prints landscape     *
//*       compressed              \NOCOMPR                normal courier       *
//*       IBM symbol set          \NOIBM                  HP symbol set        *
//*       8 lines per inch        \6LPI                   6 lines per inch     *
//*       use LPT1:               \LPT2                   use LPT2:            *
//*       bars two lines thick    \LINES                  bars one line thick  *
//*       or three lines thick                                                 *
//*       at 6lpi                                                              *
//*                                                                            *
//*       parameters can be used in any combination or order                   *
//*                                                                            *
//*       12/25/90        must be compiled with PDQ and my ADVBAS-based        *
//*                       subroutines                                          *
//*                                                                            *
//*                                                                            *
//*       Copyright 1990, Richard G. Minutillo                                 *
//*                                                                            *
//******************************************************************************

program FABPrint;

{$APPTYPE CONSOLE}

uses
  SysUtils, Windows, Printers,
    UPrnRaw;                           //Based on unit by:
                                       //Mark van der Hijden
var
  //
  //   original
  //
  ch: char;
  //
  startwid: integer;
  compressed: boolean;  //QB original was integer
  portrait: boolean;    //QB original was integer
  lpi: integer;
  sline: integer;
  lpt: integer;
  //
  // undeclared in QB but must be declared in Pascal
  //
  noibm: boolean = false;
  wid: integer;
  wd: integer;
  lengthinhalfinches: integer;
  widthinhalfinches: integer;
  linedots: integer;
  leng: integer;
  wdots: integer;
  ln: integer;
  work: string;
  header: string;
  stamp: string;
  lnum: integer;
  lnumstr: string;
  pnum: integer;
  pnumstr: string;
  //
  i, j: integer;
  newln: integer;
  devnum: integer;
  devname: string;
  fname: Tfilename;
  F: textfile;
  //
  color: boolean = false;        // ??

const
  es: char = #27;
  cr: char = #13;
  lf: char = #10;
  ff: char = #12;
  //
  // New for FindCmdLineSwitch function
  //
  pdelim: TSysCharSet = [#45,#47]; //either '-' or '/'

//
// Begin subroutines
//

function MakeElip(APath: string): string;
//
// Makes an elipsis in middle of long path string
//
var i: integer;
   send, sbegin: string;
const maxlength: integer = 30;
begin
  if POS('\',Apath) =0 then result:=''
  else if length(Apath) < maxlength then result := APath
  else begin
    i:=length(Apath)- 1;
    while copy(Apath,i,1) <> '\' do dec(i);
    send:=copy(Apath,i,length(apath) - i + 1);
    sbegin:=copy(Apath, 1, (maxlength - 2) - length(send) - 3);
    result :=sbegin + '...' + send;
    end;
end;

procedure PrintHelp;
begin
   WriteLn(lf);
   Writeln('syntax: FABPrint [-option, /option.. /p:printer /f:filename]');
   Writeln('    or  FABPrint [/listprinters or -listprinters]');
   Writeln('    or  FABPrint [/help or -help to print this screen');
   WriteLn(lf);
   Writeln('Defaults: portrait, compressed, ibm symbol set, 8lpi, wide grey bars');
   Writeln('Legal options: /land, /full, /noibm, /6lpi, /lines, /color');
   WriteLn(lf);
   Writeln('Option parameters can be used in any combination or order, and');
   Writeln('you may use upper or lower case.');
   WriteLn(lf);
   Writeln('The /listprinters option will deliver a a list of available');
   Writeln('Windows(TM) printers by full name and by number.');
   WriteLn(lf);
   Writeln('Omit printer and output will go to the Windows(TM) default printer.');
   Writeln('with /p you may specify a printer index number or printer name.');
   Writeln('If you provide a name  or part of a name FABPrint will try to find');
   Writeln('the first and closest match in the Windows(TM) printer list');
   WriteLn(lf);
   Writeln('No matter which device you use, output is ALWAYS formatted');
   Writeln('for HP' + #39 + 's PCL1 printer language. (Color is not supported.)');
   WriteLn(lf);
   Writeln('Filename parameter may contain spaces if enclosed in double quotes:');
   Writeln('  eg /f:"C:\Profram Files\My File.Ext"');
   Writeln('Omit a filename and you will be prompted; at the prompt you may enter');
   Writeln('a response containing spaces without enclosing it in double quotes.');
   WriteLn(lf);
   Writeln('The filename will be searched for in the executable directory,');
   Writeln('the current directory, the Windows(TM) and System directories,');
   Writeln('and in the search path, in that order. FABPrint will print');
   Writeln('the first file found.');
end;

procedure DumpPrinters;
begin
   WriteLn('Legal printers are as follows:');
   for i := 0 to printer.Printers.Count - 1 do begin
      Writeln(#9 + inttostr(i) + #9 + Printer.printers.strings[i]);
      end;
end;

procedure IDPrinter;
var testname: string;
begin
  for i := 1 to Paramcount do
    if (LowerCase(copy(ParamStr(i),2, 1)) = 'p')
     and (pos(':', Paramstr(i)) <> 0)then
      begin
      try
      testname := LowerCase(copy(ParamStr(i), 4, length(ParamStr(i)) - 1));
      devnum := strtoint(testname);
      devname := printer.printers.strings[devnum];
      exit;
      except
      for j := 0 to printer.printers.Count - 1 do
        if pos(testname, LowerCase(printer.printers.strings[j])) <> 0 then
          begin
          devname := printer.printers.strings[j];
          devnum := j;
          exit;
          end;
      //
      end; //try
     end; //if
end;

procedure IDFileName;
begin
  for i := 1 to Paramcount do
   if (LowerCase(copy(ParamStr(i),2, 1)) = 'f')
     and (pos(':', Paramstr(i)) <> 0)then
      begin
      fname := LowerCase(copy(ParamStr(i), 4, length(ParamStr(i)) - 1));
      exit;
      end;
  Write('Enter filename: ');
  Readln(fname);
end;

function ISFileName(tname: string): boolean;
var
  fnamerec: TSearchRec;
  FoundFileName: array[0..255] of Char;
  SearchFile: array[0..255] of char;
  Ext: array[0..3] of char;
  APtr: pchar;
begin
  Result := False;
  if SearchPath(nil,strPCopy(SearchFile, tname),
   nil, 255, @FoundFileName, Aptr) > 0 then
    begin
    Result := True;
    fname := StrPas(FoundFileName);
    end;
end;

Procedure DirectPrint(s: String);
begin
  WriteData(pchar(S), length(S));
end;

procedure ShadeIt;
var stepval, vert: integer;                             // add local variables
begin
 work := es + '&f1Y'; Directprint(work);                //specify macro ID
 work := es + '&f0X'; Directprint(work);                //start macro def
 work := es + '&f0S'; Directprint(work);                //store cursor position
 stepval := linedots * sline * 2;
 //
 if color then work := es + '*v10S'; Directprint(work);        //green ??
 //
 //define bar start & stop
 //
 //  need to make first bar manually since the for..to..step
 //     syntax not supported in pascal
 //
 work := es + '&a' + InttoStr(vert) + 'V'; Directprint(work);  //position cursor
 work := es + '*c' + InttoStr(wdots) + 'A'; Directprint(work); //set shade width
 work := es + '*c' + InttoStr(linedots * sline) + 'V';         //set shade height
 Directprint(work);
 work := es + '*c10G'; Directprint(work);                      //set shade degree
 work := es + '*c2P'; Directprint(work);
 //
 for vert := linedots * sline to (linedots * leng - 1) do // STEP stepval
  if ((vert mod stepval) = 0) then                        // substitute for STEP
   begin
   work := es + '&a' + InttoStr(vert) + 'V'; Directprint(work);
   work := es + '*c' + InttoStr(wdots) + 'A'; Directprint(work);
   work := es + '*c' + InttoStr(linedots * sline) + 'V'; Directprint(work);
   work := es + '*c10G'; Directprint(work);
   work := es + '*c2P'; Directprint(work);
   end;
 // NEXT                                                //print shade
 work := es + '&f1S'; Directprint(work);                //restore cursor position
 work := es + '&f1X'; Directprint(work);                //end macro definition
 //                                                     //enable macro as overlay
 work := es + '&f4X'; Directprint(work);                // always executes AFTER
end;                                                    // rest of page


procedure Newpage;
var factor: integer;
begin
 if portrait then factor := 1 else factor := 0;
 pnum := pnum + 1;
 pnumstr := inttostr(pnum);
 pnumstr := 'Page ' + pnumstr;
 insert(pnumstr, header, wid - length(pnumstr) + factor);
 //MID$(header$, wid - LEN(pnum$) + ABS(portrait)) = pnum$         //an adjustment for neatness
end;

procedure NewLNum;
begin
 lnum := lnum + 1;
 lnumstr := inttostr(lnum);
 lnumstr := StringofChar(#32, 6 - Length(lnumstr)) + #32 + lnumstr + ': ';
end;

//
//  main loop isolated as procedure so it can be repeated
//

procedure FABListExecute(fname: string);
begin
  //
  //  Make Sure File Exists
  //
  if not ISFileName(fname) then
    begin
    Writeln('Cannot locate ', fname, '. Sorry.');
    exit;                                            //bail out of program
    end
  else Writeln('Found: ', fname);
  //
  if (lpi = 6) AND (sline <> 1) then sline := 3;
 //
 linedots := 720 div lpi;                       //number of decipoints/per/line
 leng := (lengthinhalfinches * lpi) div 2;      //number of lines per page
 wdots := (300 * widthinhalfinches) div 2;      //shade width in dots (@300 dpi)
  //  Now do the work
  //     directly translated from the original QB code
  //     and maintaining the interger only arithmetic of PDQ
 //
 //
 //  Added for Delphi Printer Object
 //
 StartJob(printer.Printers[devnum],'FABPrint Job');
 //Printer.BeginDoc;
 //
 //start with reset
 //
 work := es + 'E'; Directprint(work);
 //
 //set landscape if required
 //
 if not portrait then work := es + '&l1O'; DirectPrint(work);
 //
 //set 8 lpi
 //
 work := es + '&l' + inttostr(lpi) + 'D'; DirectPrint(work);
 //
 //set IBM symbol set
 //
 if not noibm then work := es + '(10U'; DirectPrint(work);
 //
 //set compressed if required
 //
 if compressed then work := es + '(s0p16.66H'; DirectPrint(work);
 //
 //start build of  header line
 //
 header := StringofChar(#32, wid);
 //
 //  uses make elipsis for fname
 //
 insert('  File: ' + MakeElip(fname), header, 1);
 stamp := formatdatetime(' dddd, mmmm d, yyyy, " at " hh:mm AM/PM', Now);
 insert(stamp, header, (wid div 2) - (length(stamp) div 2));
 //MID$(header$, 1) = '  File: ' + filename$                       //insert file name
 //CALL GOODDATE(wd$, mo$, dt$)                                    //format date
 //stamp$ = wd$ + ', ' + mo$ + ' ' + dt$ + ' at ' + GOODTIME$      //format date & time stamp
 //MID$(header$, (wid \ 2) - (LEN(stamp$) \ 2)) = stamp$           //insert date and time stamp in middle

 //
 //create shading macro and invoke as overlay
 //
 ShadeIt;
 //
 //increment page & line numbers to '1'
 //
 NewPage; NewLnum;
 //
 //OPEN filename$ FOR BINARY AS 1   //open for binary to allow bit-wise reading
 //
 AssignFile(F, fname);
 Reset(F);
 //
 work := header + cr + lf; DirectPrint(work);     //print header line
 work := cr + lf; DirectPrint(work);              //skip a line
 work := lnumstr; DirectPrint(work);              //print first line number
 wd := startwid; ln := 3; newln := 0;             //reset page & width counters
                                                  //    and new line cue
 //
 //=============================================================================
 //
 //        main loop reads and prints input file one character at a time
 //
 while not EOF(F) do
  begin
  Read(F, ch);
  //GET 1, , ch
  if newln = 1 then
     begin                                //if line counter has been tripped
     if ln > leng then                    //see if we need to start
       begin                              //a new physical page
       NewPage;                                      //advance the page number
       work := header + cr + lf; DirectPrint(work);  //print header on the new page
       work := cr + lf; DirectPrint(work);           //skip a line
       ln := 3                                       //and reset the lines counter
       end;
     work := lnumstr; DirectPrint(work);             //print next line number
     wd := startwid; newln := 0;                     //reset width counter
     end;                                            //and new line cue
  //
  if ch = cr then                         //if byte is carriage return
     begin                                //we assume end of line
     Read(F, ch);                         //so we get the matching line feed
     //  GET #1, , ch
     ln := ln + 1;                        //advance page counter
     newln := 1;                          //and turnon newline cue
     NewLnum;                             //and advance the line number
     work := cr + lf; DirectPrint(work);  //and print the cr and lf
     end
  //
  else if ch = ff then
     begin                                //if the byte is a literal form feed
     ln := 1;                             //reset the page line counter
     newln := 1;                          //turn on the newline cue
     NewLnum;                             //advance the line number counter
     work := ff; DirectPrint(work);       //and print literal form feed
     end
  //
  else                                    //in all other cases
     begin                                //we print the character
     work := ch; DirectPrint(work);
     wd := wd + 1;                        //advance the line width counter
     if wd = wid then                     //if we're at the end of the line
       begin
       work := cr + lf; DirectPrint(work);//print the cr and lf
       ln := ln + 1;                      //and advance the lines counter
       if ln > leng then                  //if we're at the end
         begin                            //of the page
         NewPage;                                 //advance the page number
         work := header + cr + lf;
         DirectPrint(work);                       //print header on the new page
         work := cr + lf; DirectPrint(work);      //skip a line
         ln := 3;                                 //and reset lines counter
         work := StringofChar(#32, startwid + 4); //create blanks where the
                                                  //line number would be
         DirectPrint(work);                       //print the blanks
         wd := startwid + 5;                      //indent the width counter
         end;
                                                  //otherwise we just set the
       work := StringofChar(#32,startwid + 4);    //blanks where the
                                                  //line number would be
       DirectPrint(work);                         //print the blanks
       wd := startwid + 5;                        //indent the width counter
       end;
     end;
  // LOOP
  end; //while    end-of-loop = end-of-file
 //
 CloseFile(F);                                    //new rules
 EndPage;                                         // neatness
 EndJob;                                          // counts
//
end;  // of main procedure

//
//  begin program
//

begin
  //
  // original defaults
  //
  startwid := 8;                         //allows for line numbers and indents
  compressed := true;                    //logical flag for typeface
  portrait := true;                      //logical flag for orientation
  lpi := 8;                              //lines per inch
  sline := 2;                            //number of shaded lines
  //lpt := 1;                            //for lpt1  (no longer used)
  //
  //   New for printer ID (start with default)
  //
  devnum := printer.PrinterIndex;
  devname := printer.printers.strings[devnum];
  //
  //  Greetings
  //
  Writeln('FABPrint version 0.91 from the Fine Arts BluesBand & Poetry Press');
  Writeln('   [son of FABList]');
  //
  //  Help & Info
  //
  if FindCmdLineSwitch('listprinters', pdelim, true) then
    begin
    DumpPrinters;
    exit;
    end;
  //
  if FindCmdLineSwitch('h', pdelim, true)
    or FindCmdLineSwitch('-h', pdelim, true)
    or FindCmdLineSwitch('help', pdelim, true)
    or FindCmdLineSwitch('-help', pdelim, true)
     then
       begin
       PrintHelp;
       exit;
       end;
  //
  //  check for other options
  //
  if FindCmdLineSwitch('land', pdelim, true) then portrait := false;
  if FindCmdLineSwitch('full', pdelim, true) then compressed := false;
  if FindCmdLineSwitch('noibm', pdelim, true) then noibm := true;
  if FindCmdLineSwitch('6dpi', pdelim, true) then lpi := 6;
  if FindCmdLineSwitch('lines', pdelim, true) then sline := 1;
  if FindCmdLineSwitch('color', pdelim, true) then color := true;
  //
  //  Confirm Printer and Device Number
  //
  IDPrinter;
  Writeln('Printing on ', devname, ', printer number ', devnum);
  //
  //  Read or prompt for filename
  //
  IDFileName;
  Writeln('filename: ', fname);
  //
  if compressed and portrait then begin              //
         wid := 133;                                 // original comments
         lengthinhalfinches := 20;                   //
         widthinhalfinches := 16;
         end                                         //this is a brute force
  else if compressed AND NOT portrait then begin     //method of setting
         wid := 176;                                 //other print parameters
         lengthinhalfinches := 15;                   //using half inches to
         widthinhalfinches := 21;                    //allow legal results
         end                                         //since PDQ doesn//t allow
                                                     //floating point and so
  else if NOT compressed AND portrait then begin     //half-inches are verboten
         wid := 80;                                  //and adjustimg a few max
         lengthinhalfinches := 20;                   //parameters to compensate
         widthinhalfinches := 16;                    //for the integer divisions
         end                                         //especially in the
                                                     //landscape width, which is
  else if NOT compressed AND NOT portrait then begin //one less than the laser
         wid := 105;                                 //will print but is the same
         lengthinhalfinches := 15;                   //as the shading width
         widthinhalfinches := 21;
         end;
  //
  FABListExecute(fname);
  //
end.  // of program
